@@ -120,25 +120,37 @@ MOVETEXT_REGEX = re.compile(r"""
 SKIP_MOVETEXT_REGEX = re.compile(r""";|\{|\}""")
 
 
-CLOCK_REGEX = re.compile(r"""\[%clk\s(\d+):(\d+):(\d+(?:\.\d*)?)\]""")
-EMT_REGEX = re.compile(r"""\[%emt\s(\d+):(\d+):(\d+(?:\.\d*)?)\]""")
+CLOCK_REGEX = re.compile(r"""(?P<prefix>\s?)\[%clk\s(?P<hours>\d+):(?P<minutes>\d+):(?P<seconds>\d+(?:\.\d*)?)\](?P<suffix>\s?)""")
+EMT_REGEX = re.compile(r"""(?P<prefix>\s?)\[%emt\s(?P<hours>\d+):(?P<minutes>\d+):(?P<seconds>\d+(?:\.\d*)?)\](?P<suffix>\s?)""")
 
 EVAL_REGEX = re.compile(r"""
+    (?P<prefix>\s?)
     \[%eval\s(?:
-        \#([+-]?\d+)
-        |([+-]?(?:\d{0,10}\.\d{1,2}|\d{1,10}\.?))
+        \#(?P<mate>[+-]?\d+)
+        |(?P<cp>[+-]?(?:\d{0,10}\.\d{1,2}|\d{1,10}\.?))
     )(?:
-        ,(\d+)
+        ,(?P<depth>\d+)
     )?\]
+    (?P<suffix>\s?)
     """, re.VERBOSE)
 
 ARROWS_REGEX = re.compile(r"""
-    \[%(?:csl|cal)\s(
+    (?P<prefix>\s?)
+    \[%(?:csl|cal)\s(?P<arrows>
         [RGYB][a-h][1-8](?:[a-h][1-8])?
         (?:,[RGYB][a-h][1-8](?:[a-h][1-8])?)*
     )\]
+    (?P<suffix>\s?)
     """, re.VERBOSE)
 
+def _condense_affix(infix: str) -> Callable[[typing.Match[str]], str]:
+    def repl(match: typing.Match[str]) -> str:
+        if infix:
+            return match.group("prefix") + infix + match.group("suffix")
+        else:
+            return match.group("prefix") and match.group("suffix")
+    return repl
+
 
 TAG_ROSTER = ["Event", "Site", "Date", "Round", "White", "Black", "Result"]
 
@@ -399,8 +411,8 @@ class GameNode(abc.ABC):
 
         turn = self.turn()
 
-        if match.group(1):
-            mate = int(match.group(1))
+        if match.group("mate"):
+            mate = int(match.group("mate"))
             score: chess.engine.Score = chess.engine.Mate(mate)
             if mate == 0:
                 # Resolve this ambiguity in the specification in favor of
@@ -408,7 +420,7 @@ class GameNode(abc.ABC):
                 # who has been mated.
                 return chess.engine.PovScore(score, turn)
         else:
-            score = chess.engine.Cp(int(float(match.group(2)) * 100))
+            score = chess.engine.Cp(int(float(match.group("cp")) * 100))
 
         return chess.engine.PovScore(score if turn else -score, turn)
 
@@ -418,7 +430,7 @@ class GameNode(abc.ABC):
         this node and returns the corresponding depth, if any.
         """
         match = EVAL_REGEX.search(self.comment)
-        return int(match.group(3)) if match and match.group(3) else None
+        return int(match.group("depth")) if match and match.group("depth") else None
 
     def set_eval(self, score: Optional[chess.engine.PovScore], depth: Optional[int] = None) -> None:
         """
@@ -434,7 +446,7 @@ class GameNode(abc.ABC):
             elif score.white().mate():
                 eval = f"[%eval #{score.white().mate()}{depth_suffix}]"
 
-        self.comment, found = EVAL_REGEX.subn(eval, self.comment, count=1)
+        self.comment, found = EVAL_REGEX.subn(_condense_affix(eval), self.comment, count=1)
 
         if not found and eval:
             if self.comment and not self.comment.endswith(" "):
@@ -450,7 +462,7 @@ class GameNode(abc.ABC):
         """
         arrows = []
         for match in ARROWS_REGEX.finditer(self.comment):
-            for group in match.group(1).split(","):
+            for group in match.group("arrows").split(","):
                 arrows.append(chess.svg.Arrow.from_pgn(group))
 
         return arrows
@@ -471,7 +483,7 @@ class GameNode(abc.ABC):
                 pass
             (csl if arrow.tail == arrow.head else cal).append(arrow.pgn())  # type: ignore
 
-        self.comment = ARROWS_REGEX.sub("", self.comment).strip()
+        self.comment = ARROWS_REGEX.sub(_condense_affix(""), self.comment)
 
         prefix = ""
         if csl:
@@ -479,8 +491,10 @@ class GameNode(abc.ABC):
         if cal:
             prefix += f"[%cal {','.join(cal)}]"
 
-        if prefix:
-            self.comment = prefix + " " + self.comment if self.comment else prefix
+        if prefix and self.comment and not self.comment.startswith(" ") and not self.comment.startswith("\n"):
+            self.comment = prefix + " " + self.comment
+        else:
+            self.comment = prefix + self.comment
 
     def clock(self) -> Optional[float]:
         """
@@ -493,7 +507,7 @@ class GameNode(abc.ABC):
         match = CLOCK_REGEX.search(self.comment)
         if match is None:
             return None
-        return int(match.group(1)) * 3600 + int(match.group(2)) * 60 + float(match.group(3))
+        return int(match.group("hours")) * 3600 + int(match.group("minutes")) * 60 + float(match.group("seconds"))
 
     def set_clock(self, seconds: Optional[float]) -> None:
         """
@@ -509,10 +523,10 @@ class GameNode(abc.ABC):
             seconds_part = f"{seconds:06.3f}".rstrip("0").rstrip(".")
             clk = f"[%clk {hours:d}:{minutes:02d}:{seconds_part}]"
 
-        self.comment, found = CLOCK_REGEX.subn(clk, self.comment, count=1)
+        self.comment, found = CLOCK_REGEX.subn(_condense_affix(clk), self.comment, count=1)
 
         if not found and clk:
-            if self.comment and not self.comment.endswith(" "):
+            if self.comment and not self.comment.endswith(" ") and not self.comment.endswith("\n"):
                 self.comment += " "
             self.comment += clk
 
@@ -527,7 +541,7 @@ class GameNode(abc.ABC):
         match = EMT_REGEX.search(self.comment)
         if match is None:
             return None
-        return int(match.group(1)) * 3600 + int(match.group(2)) * 60 + float(match.group(3))
+        return int(match.group("hours")) * 3600 + int(match.group("minutes")) * 60 + float(match.group("seconds"))
 
     def set_emt(self, seconds: Optional[float]) -> None:
         """
@@ -543,10 +557,10 @@ class GameNode(abc.ABC):
             seconds_part = f"{seconds:06.3f}".rstrip("0").rstrip(".")
             emt = f"[%emt {hours:d}:{minutes:02d}:{seconds_part}]"
 
-        self.comment, found = EMT_REGEX.subn(emt, self.comment, count=1)
+        self.comment, found = EMT_REGEX.subn(_condense_affix(emt), self.comment, count=1)
 
         if not found and emt:
-            if self.comment and not self.comment.endswith(" "):
+            if self.comment and not self.comment.endswith(" ") and not self.comment.endswith("\n"):
                 self.comment += " "
             self.comment += emt
 
@@ -1146,11 +1160,11 @@ class GameBuilder(BaseVisitor[GameT]):
             # a variation. Add as a comment for the game if the comment
             # starts before any move.
             new_comment = [self.variation_stack[-1].comment, comment]
-            self.variation_stack[-1].comment = "\n".join(new_comment).strip()
+            self.variation_stack[-1].comment = " ".join(filter(None, new_comment))
         else:
             # Otherwise, it is a starting comment.
             new_comment = [self.starting_comment, comment]
-            self.starting_comment = "\n".join(new_comment).strip()
+            self.starting_comment = " ".join(filter(None, new_comment))
 
     def visit_move(self, board: chess.Board, move: chess.Move) -> None:
         self.variation_stack[-1] = self.variation_stack[-1].add_variation(move)
@@ -1584,42 +1598,43 @@ def read_game(handle: TextIO, *, Visitor: Any = GameBuilder) -> Any:
 
     # Parse movetext.
     skip_variation_depth = 0
+    fresh_line = True
     while line:
-        read_next_line = True
-
-        # Ignore comments.
-        if line.startswith("%") or line.startswith(";"):
-            line = handle.readline()
-            continue
-
-        # An empty line means the end of a game.
-        if line.isspace():
-            visitor.end_game()
-            return visitor.result()
+        if fresh_line:
+            # Ignore comments.
+            if line.startswith("%") or line.startswith(";"):
+                line = handle.readline()
+                continue
+            # An empty line means the end of a game.
+            if line.isspace():
+                visitor.end_game()
+                return visitor.result()
+        fresh_line = True
 
         for match in MOVETEXT_REGEX.finditer(line):
             token = match.group(0)
 
             if token.startswith("{"):
                 # Consume until the end of the comment.
-                line = token[1:]
+                start_index = 2 if token.startswith("{ ") else 1
+                line = token[start_index:]
+
                 comment_lines = []
                 while line and "}" not in line:
-                    comment_lines.append(line.rstrip())
+                    comment_lines.append(line)
                     line = handle.readline()
-                end_index = line.find("}")
-                comment_lines.append(line[:end_index])
-                if "}" in line:
-                    line = line[end_index:]
-                else:
-                    line = ""
+
+                if line:
+                    close_index = line.find("}")
+                    end_index = close_index - 1 if close_index > 0 and line[close_index - 1] == " " else close_index
+                    comment_lines.append(line[:end_index])
+                    line = line[close_index + 1:]
 
                 if not skip_variation_depth:
-                    visitor.visit_comment("\n".join(comment_lines).strip())
+                    visitor.visit_comment("".join(comment_lines))
 
-                # Continue with the current or the next line.
-                if line:
-                    read_next_line = False
+                # Continue with the current line.
+                fresh_line = False
                 break
             elif token == "(":
                 if skip_variation_depth:
@@ -1673,7 +1688,7 @@ def read_game(handle: TextIO, *, Visitor: Any = GameBuilder) -> Any:
                     board_stack[-1].push(move)
                 visitor.visit_board(board_stack[-1])
 
-        if read_next_line:
+        if fresh_line:
             line = handle.readline()
 
     visitor.end_game()
