@@ -577,8 +577,7 @@ class Table:
 
         black_part, white_part = tablename.split("v")
         if self.has_pawns:
-            self.pawns = {0: white_part.count("P"),
-                          1: black_part.count("P")}
+            self.pawns = [white_part.count("P"), black_part.count("P")]
             if self.pawns[1] > 0 and (self.pawns[0] == 0 or self.pawns[1] < self.pawns[0]):
                 self.pawns[0], self.pawns[1] = self.pawns[1], self.pawns[0]
         else:
@@ -1077,12 +1076,8 @@ class WdlTable(Table):
             # Used if there are only pieces.
             self.precomp: Dict[int, PairsData] = {}
             self.pieces: Dict[int, List[int]] = {}
-
-            self.factor = {0: [0 for _ in range(TBPIECES)],
-                           1: [0 for _ in range(TBPIECES)]}
-
-            self.norm = {0: [0 for _ in range(self.num)],
-                         1: [0 for _ in range(self.num)]}
+            self.factor = [[0 for _ in range(TBPIECES)] for _ in range(2)]
+            self.norm = [[0 for _ in range(self.num)] for _ in range(2)]
 
             # Used if there are pawns.
             self.files = [PawnFileData() for _ in range(4)]
@@ -1495,7 +1490,7 @@ class Tablebase:
 
     If *max_fds* is not ``None``, will at most use *max_fds* open file
     descriptors at any given time. The least recently used tables are closed,
-    if nescessary.
+    if necessary.
     """
     def __init__(self, *, max_fds: Optional[int] = 128, VariantBoard: Type[chess.Board] = chess.Board) -> None:
         self.variant = VariantBoard
